{% load currency_filters %}

<h2>{{ block.title }}</h2>

{{ block.description|safe }}

<div>
    {% for group in groups %}
        {% if group.is_visible %}
            {{ group.name }} |
        {% else %}
            <a href="">{{ group.name }}</a>
        {% endif %} 
    {% endfor %}
</div>
<div>
    {% for group in groups %}
        {% if group.is_visible %}
            {% for product in group.products %}
                <li>
                    <a href="{{ product.get_absolute_url }}">{{ product.get_title }}</a><br/>
                    {% if product.is_group %}
                        From {{ product.min_variant_price_incl_tax|currency }}
                    {% else %}
                        {% if product.has_stockrecord %} 
                            {{ product.stockrecord.price_incl_tax|currency }}<br/>
                            {{ product.stockrecord.availability }}
                        {% else %}
                            Not available    
                        {% endif %}
                    {% endif %}    
                </li>
            {% endfor %}
        {% endif %} 
    {% endfor %}
</div>
<p><a href="">See more</a></p>

