{% extends "layout.html" %}

{% load currency_filters %}

{% block header %}
<h2>Address book</h2>
{% endblock header %}


{% block content %}

<h3>Edit address</h3>
<form action="." method="post">
    {% csrf_token %}
 
    
    {{ form.as_p }}
    <input type="submit" value="Save address" />
    or
    <a href="{% url customer:address-list %}">Cancel and return to address book</a>
</form>

{% endblock content %}

