{% if promotion.image %}
{% if promotion.link_url %}
    <a href="{{ promotion.link_url }}"><img src="{{ promotion.image.url }}" title="{{ promotion.name }}" /></a>
{% else %}
    <img src="{{ promotion.image.url }}" title="{{ promotion.name }}" />
{% endif %}
{% endif %}

